@@ -19,6 +19,7 @@
 Base utilities to build API operation managers and objects on top of.
 """
 
+import abc
 import contextlib
 import copy
 import hashlib
@@ -63,7 +64,8 @@ class Manager(utils.HookableMixin):
     def api_version(self):
         return self.api.api_version
 
-    def _list(self, url, response_key, manager=None, body=None):
+    def _list(self, url, response_key, manager=None, body=None,
+              return_raw=None):
         """List the collection.
 
         :param url: a partial URL, e.g., '/shares'
@@ -96,6 +98,8 @@ class Manager(utils.HookableMixin):
                 pass
         with self.completion_cache('human_id', obj_class, mode="w"):
             with self.completion_cache('uuid', obj_class, mode="w"):
+                if return_raw:
+                    return data
                 resource = [obj_class(manager, res, loaded=True)
                             for res in data if res]
                 if 'count' in body:
@@ -125,8 +129,8 @@ class Manager(utils.HookableMixin):
         # pair
         username = cliutils.env('OS_USERNAME', 'MANILA_USERNAME')
         url = cliutils.env('OS_URL', 'MANILA_URL')
-        uniqifier = hashlib.sha1(username.encode('utf-8') +
-                                 url.encode('utf-8')).hexdigest()
+        uniqifier = hashlib.sha256(username.encode('utf-8') +
+                                   url.encode('utf-8')).hexdigest()
 
         cache_dir = os.path.expanduser(os.path.join(base_dir, uniqifier))
 
@@ -167,9 +171,11 @@ class Manager(utils.HookableMixin):
             except UnicodeEncodeError:
                 pass
 
-    def _get(self, url, response_key=None):
+    def _get(self, url, response_key, return_raw=False):
         resp, body = self.api.client.get(url)
         if response_key:
+            if return_raw:
+                return body[response_key]
             return self.resource_class(self, body[response_key], loaded=True)
         else:
             return self.resource_class(self, body, loaded=True)
@@ -192,6 +198,9 @@ class Manager(utils.HookableMixin):
             with self.completion_cache('uuid', self.resource_class, mode="a"):
                 return self.resource_class(self, body[response_key])
 
+    def _accept(self, url, body):
+        resp, body = self.api.client.post(url, body=body)
+
     def _delete(self, url):
         resp, body = self.api.client.delete(url)
 
@@ -354,3 +363,150 @@ class Resource(object):
 
     def to_dict(self):
         return copy.deepcopy(self._info)
+
+
+class MetadataCapableResource(Resource, metaclass=abc.ABCMeta):
+
+    superresource = None
+
+    def _get_subresource_and_resource(self, superresource):
+        resource = self
+        subresource = None
+        superresource = superresource or self.superresource
+        if superresource is not None:
+            resource = superresource
+            subresource = self
+        return resource, subresource
+
+    def get_metadata(self, superresource=None):
+        """Get metadata of a resource
+
+        :param superresource: either a parent resource object or text with
+            its ID. Required for sub-resources such as share export
+            locations which do not include a reference to the parent object
+            by default
+        """
+
+        resource, subresource = self._get_subresource_and_resource(
+            superresource)
+
+        return self.manager.get_metadata(resource, subresource=subresource)
+
+    def set_metadata(self, metadata, superresource=None):
+        """Set or update metadata for the resource.
+
+        :param metadata: A dictionary of key:value pairs to be set as
+            resource metadata
+        :param superresource: either a parent resource object or text with
+            its ID. Required for sub-resources such as share share export
+            locations which do not include a reference to the parent object
+            by default
+        """
+        resource, subresource = self._get_subresource_and_resource(
+            superresource)
+
+        return self.manager.set_metadata(resource, metadata,
+                                         subresource=subresource)
+
+    def delete_metadata(self, keys, superresource=None):
+        """Delete specified keys from the given resource.
+
+        :param keys: An iterable with keys of metadata items to be deleted
+        :param superresource: either a parent resource object or text with
+            its ID. Required for sub-resources such as share share export
+            locations which do not include a reference to the parent object
+            by default
+        """
+        resource, subresource = self._get_subresource_and_resource(
+            superresource)
+
+        return self.manager.delete_metadata(resource, keys,
+                                            subresource=subresource)
+
+    def update_all_metadata(self, metadata, superresource=None):
+        """Update all metadata for this resource.
+
+        :param metadata: A dictionary of key:value pairs of resource metadata
+            to be updated
+        :param superresource: either a parent resource object or text with
+            its ID. Required for sub-resources such as share share export
+            locations which do not include a reference to the parent object
+            by default
+        """
+        resource, subresource = self._get_subresource_and_resource(
+            superresource)
+
+        return self.manager.update_all_metadata(resource,
+                                                metadata,
+                                                subresource=subresource)
+
+
+class MetadataCapableManager(ManagerWithFind, metaclass=abc.ABCMeta):
+    """Provides extended behavior to objects to handle key=value metadata."""
+
+    resource_path = None
+    subresource_path = None
+
+    def get_metadata(self, resource, subresource=None):
+        """Get metadata of a resource.
+
+        :param resource: either resource object or text with its ID.
+        :param subresource: either a child resource object or text with its ID
+        """
+        resource = getid(resource)
+        if subresource:
+            subresource = getid(subresource)
+            resource = f"{resource}{self.subresource_path}/{subresource}"
+
+        return self._get(f"{self.resource_path}/{resource}/metadata",
+                         "metadata")
+
+    def set_metadata(self, resource, metadata, subresource=None):
+        """Set or update metadata for resource.
+
+        :param resource: either resource object or text with its ID.
+        :param metadata: A dictionary of key:value pairs to be set as
+            resource metadata
+        :param subresource: either a child resource object or text with its ID
+        """
+        body = {'metadata': metadata}
+        resource = getid(resource)
+        if subresource:
+            subresource = getid(subresource)
+            resource = f"{resource}{self.subresource_path}/{subresource}"
+
+        return self._create(f"{self.resource_path}/{resource}/metadata",
+                            body,
+                            "metadata")
+
+    def delete_metadata(self, resource, keys, subresource=None):
+        """Delete specified keys from resource metadata.
+
+        :param resource: either resource object or text with its ID.
+        :param keys: An iterable with keys of metadata items to be deleted
+        :param subresource: either a child resource object or text with its ID
+        """
+        resource = getid(resource)
+        if subresource:
+            subresource = getid(subresource)
+            resource = f"{resource}{self.subresource_path}/{subresource}"
+
+        for key in keys:
+            self._delete(f"{self.resource_path}/{resource}/metadata/{key}")
+
+    def update_all_metadata(self, resource, metadata, subresource=None):
+        """Update all metadata of a resource.
+
+        :param resource: either resource object or text with its ID.
+        :param metadata: A dictionary of key:value pairs of resource metadata
+            to be updated
+        :param subresource: either a child resource object or text with its ID
+        """
+        body = {'metadata': metadata}
+        resource = getid(resource)
+        if subresource:
+            subresource = getid(subresource)
+            resource = f"{resource}{self.subresource_path}/{subresource}"
+
+        return self._update(f"{self.resource_path}/{resource}/metadata",
+                            body)
