@@ -25,8 +25,6 @@ import textwrap
 from oslo_utils import encodeutils
 from oslo_utils import strutils
 import prettytable
-import six
-from six import moves
 
 from manilaclient.common._i18n import _
 
@@ -55,7 +53,7 @@ def validate_args(fn, *args, **kwargs):
     :param arg: the positional arguments supplied
     :param kwargs: the keyword arguments supplied
     """
-    argspec = inspect.getargspec(fn)
+    argspec = inspect.getfullargspec(fn)
 
     num_defaults = len(argspec.defaults or [])
     required_args = argspec.args[:len(argspec.args) - num_defaults]
@@ -178,10 +176,7 @@ def print_list(objs, fields, formatters=None, sortby_index=0,
                 row.append(data)
         pt.add_row(row)
 
-    if six.PY3:
-        print(encodeutils.safe_encode(pt.get_string(**kwargs)).decode())
-    else:
-        print(encodeutils.safe_encode(pt.get_string(**kwargs)))
+    print(encodeutils.safe_encode(pt.get_string(**kwargs)).decode())
 
 
 def print_dict(dct, dict_property="Property", wrap=0):
@@ -196,12 +191,12 @@ def print_dict(dct, dict_property="Property", wrap=0):
     for k, v in dct.items():
         # convert dict to str to check length
         if isinstance(v, dict):
-            v = six.text_type(v)
+            v = str(v)
         if wrap > 0:
-            v = textwrap.fill(six.text_type(v), wrap)
+            v = textwrap.fill(str(v), wrap)
         # if value has a newline, add in multiple rows
         # e.g. fault with stacktrace
-        if v and isinstance(v, six.string_types) and r'\n' in v:
+        if v and isinstance(v, str) and r'\n' in v:
             lines = v.strip().split(r'\n')
             col1 = k
             for line in lines:
@@ -210,10 +205,7 @@ def print_dict(dct, dict_property="Property", wrap=0):
         else:
             pt.add_row([k, v])
 
-    if six.PY3:
-        print(encodeutils.safe_encode(pt.get_string()).decode())
-    else:
-        print(encodeutils.safe_encode(pt.get_string()))
+    print(encodeutils.safe_encode(pt.get_string()).decode())
 
 
 def get_password(max_password_prompts=3):
@@ -223,7 +215,7 @@ def get_password(max_password_prompts=3):
     if hasattr(sys.stdin, "isatty") and sys.stdin.isatty():
         # Check for Ctrl-D
         try:
-            for __ in moves.range(max_password_prompts):
+            for __ in range(max_password_prompts):
                 pw1 = getpass.getpass("OS Password: ")
                 if verify:
                     pw2 = getpass.getpass("Please verify: ")
@@ -259,8 +251,8 @@ def get_service_type(f):
     return getattr(f, 'service_type', None)
 
 
-def pretty_choice_list(l):
-    return ', '.join("'%s'" % i for i in l)
+def pretty_choice_list(choices):
+    return ', '.join("'%s'" % choice for choice in choices)
 
 
 def exit(msg=''):
@@ -269,19 +261,18 @@ def exit(msg=''):
     sys.exit(1)
 
 
-def transform_export_locations_to_string_view(export_locations):
-    export_locations_string_view = ''
-    replica_export_location_ignored_keys = (
-        'replica_state', 'availability_zone', 'share_replica_id')
-    for el in export_locations:
-        if hasattr(el, '_info'):
-            export_locations_dict = el._info
+def convert_dict_list_to_string(data, ignored_keys=None):
+    ignored_keys = ignored_keys or []
+    if not isinstance(data, list):
+        data = [data]
+    data_string = ''
+    for datum in data:
+        if hasattr(datum, '_info'):
+            datum_dict = datum._info
         else:
-            export_locations_dict = el
-        for k, v in export_locations_dict.items():
-            # NOTE(gouthamr): We don't want to show replica related info
-            # twice in the output, so ignore those.
-            if k not in replica_export_location_ignored_keys:
-                export_locations_string_view += '\n%(k)s = %(v)s' % {
+            datum_dict = datum
+        for k, v in datum_dict.items():
+            if k not in ignored_keys:
+                data_string += '\n%(k)s = %(v)s' % {
                     'k': k, 'v': v}
-    return export_locations_string_view
+    return data_string
