@@ -16,11 +16,11 @@
 
 from manilaclient import api_versions
 from manilaclient import base
-from manilaclient.common.apiclient import base as common_base
 from manilaclient.common import constants
 
 
-class ShareSnapshot(common_base.Resource):
+class ShareSnapshot(base.MetadataCapableResource):
+
     """Represent a snapshot of a share."""
 
     def __repr__(self):
@@ -58,11 +58,13 @@ class ShareSnapshot(common_base.Resource):
         return self.manager.access_list(self)
 
 
-class ShareSnapshotManager(base.ManagerWithFind):
+class ShareSnapshotManager(base.MetadataCapableManager):
     """Manage :class:`ShareSnapshot` resources."""
     resource_class = ShareSnapshot
+    resource_path = '/snapshots'
 
-    def create(self, share, force=False, name=None, description=None):
+    def _do_create(self, share, force=False, name=None, description=None,
+                   metadata=None):
         """Create a snapshot of the given share.
 
         :param share_id: The ID of the share to snapshot.
@@ -70,14 +72,27 @@ class ShareSnapshotManager(base.ManagerWithFind):
                       share is busy. Default is False.
         :param name: Name of the snapshot
         :param description: Description of the snapshot
+        :param metadata: dict - optional metadata to set on share creation
         :rtype: :class:`ShareSnapshot`
         """
-        body = {'snapshot': {'share_id': common_base.getid(share),
+
+        metadata = metadata if metadata is not None else dict()
+        body = {'snapshot': {'share_id': base.getid(share),
                              'force': force,
                              'name': name,
-                             'description': description}}
+                             'description': description,
+                             'metadata': metadata}}
         return self._create('/snapshots', body, 'snapshot')
 
+    @api_versions.wraps("2.0", "2.72")
+    def create(self, share, force=False, name=None, description=None):
+        return self._do_create(share, force, name, description)
+
+    @api_versions.wraps("2.73")
+    def create(self, share, force=False, name=None, description=None,# noqa F811
+               metadata=None):
+        return self._do_create(share, force, name, description, metadata)
+
     @api_versions.wraps("2.12")
     def manage(self, share, provider_location,
                driver_options=None,
@@ -93,7 +108,7 @@ class ShareSnapshotManager(base.ManagerWithFind):
         """
         driver_options = driver_options if driver_options else {}
         body = {
-            'share_id': common_base.getid(share),
+            'share_id': base.getid(share),
             'provider_location': provider_location,
             'driver_options': driver_options,
             'name': name,
@@ -117,7 +132,7 @@ class ShareSnapshotManager(base.ManagerWithFind):
             of snapshot to delete.
         :rtype: :class:`ShareSnapshot`
         """
-        snapshot_id = common_base.getid(snapshot)
+        snapshot_id = base.getid(snapshot)
         return self._get('/snapshots/%s' % snapshot_id, 'snapshot')
 
     def list(self, detailed=True, search_opts=None, sort_key=None,
@@ -161,11 +176,11 @@ class ShareSnapshotManager(base.ManagerWithFind):
 
         :param snapshot: The :class:`ShareSnapshot` to delete.
         """
-        self._delete("/snapshots/%s" % common_base.getid(snapshot))
+        self._delete("/snapshots/%s" % base.getid(snapshot))
 
     def _do_force_delete(self, snapshot, action_name="force_delete"):
         """Delete the specified snapshot ignoring its current state."""
-        return self._action(action_name, common_base.getid(snapshot))
+        return self._action(action_name, base.getid(snapshot))
 
     @api_versions.wraps("1.0", "2.6")
     def force_delete(self, snapshot):
@@ -186,7 +201,7 @@ class ShareSnapshotManager(base.ManagerWithFind):
             return
 
         body = {'snapshot': kwargs, }
-        snapshot_id = common_base.getid(snapshot)
+        snapshot_id = base.getid(snapshot)
         return self._update("/snapshots/%s" % snapshot_id, body)
 
     def _do_reset_state(self, snapshot, state, action_name="reset_status"):
@@ -222,7 +237,7 @@ class ShareSnapshotManager(base.ManagerWithFind):
         return self._do_deny(snapshot, id)
 
     def _do_access_list(self, snapshot):
-        snapshot_id = common_base.getid(snapshot)
+        snapshot_id = base.getid(snapshot)
         access_list = self._list("/snapshots/%s/access-list" % snapshot_id,
                                  'snapshot_access_list')
         return access_list
@@ -235,5 +250,5 @@ class ShareSnapshotManager(base.ManagerWithFind):
         """Perform a snapshot 'action'."""
         body = {action: info}
         self.run_hooks('modify_body_for_action', body, **kwargs)
-        url = '/snapshots/%s/action' % common_base.getid(snapshot)
+        url = '/snapshots/%s/action' % base.getid(snapshot)
         return self.api.client.post(url, body=body)
